@@ -1,6 +1,7 @@
 /*
  * Remmina - The GTK Remote Desktop Client
- * Copyright (C) 2014-2021 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2014-2023 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2023-2024 Hiroyuki Tanaka, Sunil Bhat
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,11 +32,20 @@
  *  files in the program, then also delete it here.
  *
  */
+#include <curl/curl.h>
+#include <gdk/gdk.h>
 
+#define G_LOG_USE_STRUCTURED
+#ifndef G_LOG_DOMAIN
+#define G_LOG_DOMAIN    ((gchar*)"remmina")
+#endif  /* G_LOG_DOMAIN */
+#ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
+#elif defined(GDK_WINDOWING_WAYLAND)
+#include <gdk/gdkwayland.h>
+#endif
 #include <gio/gio.h>
 #include <glib/gi18n.h>
-#include <gtk/gtk.h>
 #include <stdlib.h>
 
 #include "config.h"
@@ -44,6 +54,7 @@
 #include "remmina_exec.h"
 #include "remmina_file_manager.h"
 #include "remmina_icon.h"
+#include "remmina_log.h"
 #include "remmina_main.h"
 #include "remmina_masterthread_exec.h"
 #include "remmina_plugin_manager.h"
@@ -57,9 +68,7 @@
 #include "remmina_ssh_plugin.h"
 #include "remmina_widget_pool.h"
 #include "remmina/remmina_trace_calls.h"
-#include "rmnews.h"
-#include "remmina_stats_sender.h"
-
+#include "remmina_info.h"
 
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
@@ -79,17 +88,24 @@ static int gcrypt_thread_initialized = 0;
 #endif  /* HAVE_LIBGCRYPT */
 
 gboolean kioskmode;
+gboolean imode;
+gboolean disablenews;
+gboolean disablestats;
+gboolean disabletoolbar;
+gboolean fullscreen;
+gboolean extrahardening;
+gboolean disabletrayicon;
 
 static GOptionEntry remmina_options[] =
 {
 	// TRANSLATORS: Shown in terminal. Do not use characters that may be not supported on a terminal
 	{ "about",	      'a',  0,			  G_OPTION_ARG_NONE,	       NULL, N_("Show \'About\'"),								     NULL	},
 	// TRANSLATORS: Shown in terminal. Do not use characters that may be not supported on a terminal
-	{ "connect",	      'c',  0,			  G_OPTION_ARG_FILENAME_ARRAY, NULL, N_("Connect either to a desktop described in a file (.remmina or a filetype supported by a plugin) or a supported URI (RDP, VNC, SSH or SPICE)"),	     N_("FILE")	},
+	{ "connect",	      'c',  0,			  G_OPTION_ARG_FILENAME_ARRAY, NULL, N_("Connect either to a desktop described in a file (.remmina or a filetype supported by a plugin) or a supported URI (RDP, VNC, SSH, HTTP, HTTPS or SPICE)"),	     N_("FILE")	},
 	// TRANSLATORS: Shown in terminal. Do not use characters that may be not supported on a terminal
-	{ G_OPTION_REMAINING, '\0', 0,			  G_OPTION_ARG_FILENAME_ARRAY, NULL, N_("Connect to a desktop described in a file (.remmina or a type supported by a plugin)"),	     N_("FILE")	},
+	{ G_OPTION_REMAINING, '\0', 0,			  G_OPTION_ARG_FILENAME_ARRAY, NULL, N_("Connect to a desktop described in a file (.remmina or a filetype supported by a plugin)"),	     N_("FILE")	},
 	// TRANSLATORS: Shown in terminal. Do not use characters that may be not supported on a terminal
-	{ "edit",	      'e',  0,			  G_OPTION_ARG_FILENAME_ARRAY, NULL, N_("Edit desktop connection described in file (.remmina or type supported by plugin)"), N_("FILE")	},
+	{ "edit",	      'e',  0,			  G_OPTION_ARG_FILENAME_ARRAY, NULL, N_("Edit desktop connection described in file (.remmina or a filetype supported by plugin)"), N_("FILE")	},
 	{ "help",	      '?',  G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE,	       NULL, NULL,										     NULL	},
 	// TRANSLATORS: Shown in terminal. Do not use characters that may be not supported on a terminal
 	{ "kiosk",	      'k',  0,			  G_OPTION_ARG_NONE,	       NULL, N_("Start in kiosk mode"),								     NULL	},
@@ -119,6 +135,12 @@ static GOptionEntry remmina_options[] =
 	// TRANSLATORS: Shown in terminal. Do not use characters that may be not supported on a terminal
 	{ "set-option",	      0,    0,			  G_OPTION_ARG_STRING_ARRAY,   NULL, N_("Set one or more profile settings, to be used with --update-profile"),		     NULL	},
 	{ "encrypt-password", 0,    0,			  G_OPTION_ARG_NONE,	       NULL, N_("Encrypt a password"),												  NULL		 },
+	{ "disable-news",     0,    0,            G_OPTION_ARG_NONE,           NULL, N_("Disable news"),                                                NULL           },
+	{ "disable-stats",    0,    0,            G_OPTION_ARG_NONE,           NULL, N_("Disable stats"),                                                NULL           },
+	{ "disable-toolbar", 0,    0,			  G_OPTION_ARG_NONE,	       NULL, N_("Disable toolbar"),												  NULL		 },
+	{ "enable-fullscreen", 0,    0,			  G_OPTION_ARG_NONE,	       NULL, N_("Enable fullscreen"),												  NULL		 },
+	{ "enable-extra-hardening", 0,    0,		  G_OPTION_ARG_NONE,	       NULL, N_("Enable extra hardening (disable closing confirmation, disable unsafe shortcut keys, hide tabs, hide search bar)"),	  NULL		 },
+	{ "no-tray-icon", 0,    0,			  G_OPTION_ARG_NONE,	       NULL, N_("Disable tray icon"),												  NULL		 },
 	{ NULL }
 };
 
@@ -150,10 +172,34 @@ static gint remmina_on_command_line(GApplication *app, GApplicationCommandLine *
 #if SODIUM_VERSION_INT >= 90200
 	remmina_sodium_init();
 #endif
-	remmina_pref_init();
-
 	opts = g_application_command_line_get_options_dict(cmdline);
 
+	if (g_variant_dict_lookup_value(opts, "disable-news", NULL)) {
+		disablenews = TRUE;
+	}
+
+	if (g_variant_dict_lookup_value(opts, "disable-stats", NULL)) {
+		disablestats = TRUE;
+	}
+
+	if (g_variant_dict_lookup_value(opts, "disable-toolbar", NULL)) {
+		disabletoolbar = TRUE;
+	}
+
+	if (g_variant_dict_lookup_value(opts, "enable-fullscreen", NULL)) {
+		fullscreen = TRUE;
+	}
+
+	if (g_variant_dict_lookup_value(opts, "enable-extra-hardening", NULL)) {
+		extrahardening = TRUE;
+	}
+
+	if (g_variant_dict_lookup_value(opts, "no-tray-icon", NULL)) {
+		disabletrayicon = TRUE;
+	}
+
+	remmina_pref_init();
+
 	if (g_variant_dict_lookup_value(opts, "quit", NULL)) {
 		remmina_exec_command(REMMINA_COMMAND_EXIT, NULL);
 		executed = TRUE;
@@ -161,6 +207,7 @@ static gint remmina_on_command_line(GApplication *app, GApplicationCommandLine *
 	}
 
 	if (g_variant_dict_lookup_value(opts, "about", NULL)) {
+		imode = TRUE;
 		remmina_exec_command(REMMINA_COMMAND_ABOUT, NULL);
 		executed = TRUE;
 	}
@@ -184,18 +231,18 @@ static gint remmina_on_command_line(GApplication *app, GApplicationCommandLine *
 	}
 
 	if (g_variant_dict_lookup(opts, "edit", "^aay", &files)) {
+		imode = TRUE;
 		if (files)
 			for (gint i = 0; files[i]; i++) {
 				g_debug ("Editing file: %s", files[i]);
 				remmina_exec_command(REMMINA_COMMAND_EDIT, files[i]);
 			}
-		//remmina_exec_command(REMMINA_COMMAND_EDIT, str);
-		//g_free(str);
 		executed = TRUE;
 	}
 
 	if (g_variant_dict_lookup_value(opts, "kiosk", NULL)) {
 		kioskmode = TRUE;
+		imode = TRUE;
 		remmina_exec_command(REMMINA_COMMAND_MAIN, NULL);
 		executed = TRUE;
 	}
@@ -215,6 +262,7 @@ static gint remmina_on_command_line(GApplication *app, GApplicationCommandLine *
 	}
 
 	if (g_variant_dict_lookup(opts, "pref", "&s", &str)) {
+		imode = TRUE;
 		remmina_exec_command(REMMINA_COMMAND_PREF, str);
 		executed = TRUE;
 	}
@@ -251,7 +299,6 @@ static void remmina_on_startup(GApplication *app)
 	remmina_sftp_plugin_register();
 	remmina_ssh_plugin_register();
 	remmina_icon_init();
-
 	g_set_application_name("Remmina");
 	gtk_window_set_default_icon_name(REMMINA_APP_ID);
 
@@ -262,17 +309,24 @@ static void remmina_on_startup(GApplication *app)
 	gtk_icon_theme_append_search_path(gtk_icon_theme_get_default(),
 					  REMMINA_RUNTIME_DATADIR G_DIR_SEPARATOR_S "icons");
 	g_application_hold(app);
+	remmina_info_schedule();
 
-	remmina_stats_sender_schedule();
-	rmnews_schedule();
+	gchar *log_filename = g_build_filename(g_get_tmp_dir(), LOG_FILE_NAME, NULL);
+	FILE *log_file = fopen(log_filename, "w"); // Flush log file
+	g_free(log_filename);
+
+	if (log_file != NULL) {
+		fclose(log_file);
+	}
 
 	/* Check for secret plugin and service initialization and show console warnings if
 	 * something is missing */
+	remmina_plugin_manager_get_available_plugins();
 	secret_plugin = remmina_plugin_manager_get_secret_plugin();
 	if (!secret_plugin)
 		g_print("Warning: Remmina is running without a secret plugin. Passwords will be saved in a less secure way.\n");
 	else
-		if (!secret_plugin->is_service_available())
+		if (!secret_plugin->is_service_available(secret_plugin))
 			g_print("Warning: Remmina is running with a secrecy plugin, but it cannot connect to a secrecy service.\n");
 
 	remmina_exec_command(REMMINA_COMMAND_AUTOSTART, NULL);
@@ -326,6 +380,22 @@ int main(int argc, char *argv[])
 
 	g_unsetenv("GDK_CORE_DEVICE_EVENTS");
 
+	// Checking for environment variable "G_MESSAGES_DEBUG"
+	// Give the less familiar with GLib a tip on where to get
+	// more debugging info.
+	if(!getenv("G_MESSAGES_DEBUG")) {
+		/* TRANSLATORS:
+		 * This link should point to a resource explaining how to get Remmina
+		 * to log more verbose statements.
+		 */
+		g_message(_("Remmina does not log all output statements. "
+			    "Turn on more verbose output by using "
+			    "\"G_MESSAGES_DEBUG=remmina\" as an environment variable.\n"
+			    "More info available on the Remmina wiki at:\n"
+			    "https://gitlab.com/Remmina/Remmina/-/wikis/Usage/Remmina-debugging"
+		));
+	}
+
 	/* Enable wayland backend only after GTK 3.22.27 or the clipboard
 	 * will not work. See GTK bug 790031 */
 	if (remmina_gtk_check_version(3, 22, 27))
@@ -354,9 +424,9 @@ int main(int argc, char *argv[])
 #endif  /* !HAVE_LIBGCRYPT */
 
 	/* Initialize some Remmina parts needed also on a local instance for correct handle-local-options */
+	curl_global_init(CURL_GLOBAL_ALL);
 	remmina_pref_init();
 	remmina_file_manager_init();
-
 	remmina_plugin_manager_init();
 
 
@@ -375,6 +445,7 @@ int main(int argc, char *argv[])
 	g_application_set_option_context_parameter_string (G_APPLICATION(app), _("- or protocol://username:encryptedpassword@host:port"));
 	// TRANSLATORS: Shown in terminal. Do not use characters that may be not supported on a terminal
 	g_application_set_option_context_description (G_APPLICATION(app),
+			// xgettext:no-c-format
 			_("Examples:\n"
 				"To connect using an existing connection profile, use:\n"
 				"\n"
@@ -388,10 +459,16 @@ int main(int argc, char *argv[])
 				"\tremmina -c vnc://server?VncUsername=username\n"
 				"\tremmina -c ssh://user@server\n"
 				"\tremmina -c spice://server\n"
+				"\tremmina -c https://example.com\n"
+				"\n"
+				"To quick connect and name the connection\n"
+				"\n"
+				"\tremmina -c ssh://user@server\%connection_name\n"
 				"\n"
 				"To quick connect using a URI along with an encrypted password:\n"
 				"\n"
 				"\tremmina -c rdp://username:encrypted-password@server\n"
+				"\n"
 				"\tremmina -c vnc://username:encrypted-password@server\n"
 				"\tremmina -c vnc://server?VncUsername=username\\&VncPassword=encrypted-password\n"
 				"\n"
