@@ -4,6 +4,7 @@
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
  * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  * Copyright (C) 2022-2023 Antenore Gatta, Giovanni Panozzo, Hiroyuki Tanaka
+ * Copyright (C) 2023-2024 Hiroyuki Tanaka, Sunil Bhat
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,12 +53,17 @@
 #include "remmina_public.h"
 #include "remmina_string_array.h"
 #include "remmina_pref.h"
+#include "remmina_utils.h"
 #include "remmina_log.h"
 #include "remmina/remmina_trace_calls.h"
 
-const gchar *default_resolutions = "640x480,800x600,1024x768,1152x864,1280x960,1400x1050";
+const gchar *default_resolutions = "640x480,800x600,1024x768,1152x864,1280x960,1400x1050,1920x1080";
 const gchar *default_keystrokes = "Send hello world§hello world\\n";
 
+extern gboolean info_disable_stats;
+extern gboolean info_disable_news;
+extern gboolean info_disable_tip;
+
 gchar *remmina_keymap_file;
 static GHashTable *remmina_keymap_table = NULL;
 
@@ -217,11 +223,14 @@ void remmina_pref_file_load_colors(GKeyFile *gkeyfile, RemminaColorPref *color_p
 	}
 }
 
+extern gboolean disablenews;
+extern gboolean disablestats;
+
 void remmina_pref_init(void)
 {
 	TRACE_CALL(__func__);
 	GKeyFile *gkeyfile;
-	gchar *remmina_dir;
+	gchar *remmina_dir, *remmina_pref_path, *user_config_path;
 	const gchar *filename = "remmina.pref";
 	const gchar *colors_filename = "remmina.colors";
 	g_autofree gchar *remmina_colors_file = NULL;
@@ -239,10 +248,13 @@ void remmina_pref_init(void)
 	remmina_dir = g_build_path("/", g_get_home_dir(), legacy, NULL);
 	if (g_file_test(remmina_dir, G_FILE_TEST_IS_DIR)) {
 		dir = g_dir_open(remmina_dir, 0, NULL);
-		remmina_pref_file_do_copy(
-			g_build_path("/", remmina_dir, filename, NULL),
-			g_build_path("/", g_get_user_config_dir(),
-				     "remmina", filename, NULL));
+		remmina_pref_path = g_build_path("/", remmina_dir, filename, NULL);
+		user_config_path = g_build_path("/", g_get_user_config_dir(),
+				     "remmina", filename, NULL);
+		remmina_pref_file_do_copy(remmina_pref_path, user_config_path);
+		g_dir_close(dir);
+		g_free(remmina_pref_path);
+		g_free(user_config_path);
 	}
 
 	/* /usr/local/etc/remmina */
@@ -254,15 +266,22 @@ void remmina_pref_init(void)
 		if (g_file_test(remmina_dir, G_FILE_TEST_IS_DIR)) {
 			dir = g_dir_open(remmina_dir, 0, NULL);
 			while ((filename = g_dir_read_name(dir)) != NULL) {
-				remmina_pref_file_do_copy(
-					g_build_path("/", remmina_dir, filename, NULL),
-					g_build_path("/", g_get_user_config_dir(),
-						     "remmina", filename, NULL));
+				remmina_pref_path = g_build_path("/", remmina_dir, filename, NULL);
+				user_config_path = g_build_path("/", g_get_user_config_dir(),
+							"remmina", filename, NULL);
+				remmina_pref_file_do_copy(remmina_pref_path, user_config_path);
+				g_free(remmina_pref_path);
+				g_free(user_config_path);
 			}
 			g_free(remmina_dir), remmina_dir = NULL;
+			g_dir_close(dir);
+		} else if (dirs[i + 1] != NULL) {
+			// Handle the case where g_file_test(remmina_dir, G_FILE_TEST_IS_DIR) is NULL
+			g_free(remmina_dir);
+			remmina_dir = NULL;
 		}
 	}
-
+	
 	/* The last case we use  the home ~/.config/remmina */
 	if (remmina_dir != NULL)
 		g_free(remmina_dir), remmina_dir = NULL;
@@ -275,6 +294,8 @@ void remmina_pref_init(void)
 
 	remmina_keymap_file = g_strdup_printf("%s/remmina.keymap", remmina_dir);
 
+	g_free(remmina_dir);
+
 	gkeyfile = g_key_file_new();
 
 	g_key_file_load_from_file(gkeyfile, remmina_pref_file, G_KEY_FILE_NONE, NULL);
@@ -350,6 +371,11 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.floating_toolbar_placement = FLOATING_TOOLBAR_PLACEMENT_TOP;
 
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "floating_toolbar_monitor", NULL))
+		remmina_pref.floating_toolbar_monitor = g_key_file_get_integer(gkeyfile, "remmina_pref", "floating_toolbar_monitor", NULL);
+	else
+		remmina_pref.floating_toolbar_monitor = remmina_get_primary_monitor_num();
+
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "prevent_snap_welcome_message", NULL))
 		remmina_pref.prevent_snap_welcome_message = g_key_file_get_boolean(gkeyfile, "remmina_pref", "prevent_snap_welcome_message", NULL);
 	else
@@ -378,6 +404,27 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.always_show_notes = FALSE;
 
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "mp_left", NULL))
+		remmina_pref.mp_left = g_key_file_get_boolean(gkeyfile, "remmina_pref", "mp_left", NULL);
+	else
+		remmina_pref.mp_left = FALSE;
+
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "start_fullscreen", NULL))
+		remmina_pref.start_fullscreen = g_key_file_get_boolean(gkeyfile, "remmina_pref", "start_fullscreen", NULL);
+	else
+		remmina_pref.start_fullscreen = FALSE;
+
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "start_dynres", NULL))
+		remmina_pref.start_dynres = g_key_file_get_boolean(gkeyfile, "remmina_pref", "start_dynres", NULL);
+	else
+		remmina_pref.start_dynres = FALSE;
+
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "toolbar_fix_position_multimon", NULL))
+		remmina_pref.toolbar_fix_position_multimon = g_key_file_get_boolean(gkeyfile, "remmina_pref", "toolbar_fix_position_multimon", NULL);
+	else
+		remmina_pref.toolbar_fix_position_multimon = TRUE;
+		
+
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "hide_connection_toolbar", NULL))
 		remmina_pref.hide_connection_toolbar = g_key_file_get_boolean(gkeyfile, "remmina_pref",
 									      "hide_connection_toolbar", NULL);
@@ -482,10 +529,18 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.datadir_path = g_strdup("");
 
-	if (g_key_file_has_key(gkeyfile, "remmina_pref", "remmina_file_name", NULL))
-		remmina_pref.remmina_file_name = g_key_file_get_string(gkeyfile, "remmina_pref", "remmina_file_name", NULL);
-	else
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "remmina_file_name", NULL)) {
+		if (strcmp(g_key_file_get_string(gkeyfile, "remmina_pref", "remmina_file_name", NULL), "") != 0) {
+			remmina_pref.remmina_file_name = g_key_file_get_string(gkeyfile, "remmina_pref", "remmina_file_name", NULL);
+		}
+		else {
+			REMMINA_DEBUG("remmina_file_name in config is empty, setting to %%G_%%P_%%N_%%h");
+			remmina_pref.remmina_file_name = g_strdup("%G_%P_%N_%h");
+		}
+	}
+	else {
 		remmina_pref.remmina_file_name = g_strdup("%G_%P_%N_%h");
+	}
 
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "screenshot_path", NULL)) {
 		remmina_pref.screenshot_path = g_key_file_get_string(gkeyfile, "remmina_pref", "screenshot_path", NULL);
@@ -597,6 +652,11 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.fullscreen_toolbar_visibility = FLOATING_TOOLBAR_VISIBILITY_PEEKING;
 
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "fullscreen_toolbar_delay", NULL))
+		remmina_pref.fullscreen_toolbar_delay = g_key_file_get_integer(gkeyfile, "remmina_pref", "fullscreen_toolbar_delay", NULL);
+	else
+		remmina_pref.fullscreen_toolbar_delay = 0;
+
 	if (disabletoolbar)
 		remmina_pref.fullscreen_toolbar_visibility = FLOATING_TOOLBAR_VISIBILITY_DISABLE;
 
@@ -639,6 +699,11 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.shortcutkey_scale = GDK_KEY_s;
 
+	if (g_key_file_has_key(gkeyfile, "remmina_pref", "shortcutkey_clipboard", NULL))
+		remmina_pref.shortcutkey_clipboard = g_key_file_get_integer(gkeyfile, "remmina_pref", "shortcutkey_clipboard", NULL);
+	else
+		remmina_pref.shortcutkey_clipboard = GDK_KEY_b;
+
 	if (g_key_file_has_key(gkeyfile, "remmina_pref", "shortcutkey_viewonly", NULL))
 		remmina_pref.shortcutkey_viewonly = g_key_file_get_integer(gkeyfile, "remmina_pref", "shortcutkey_viewonly", NULL);
 	else
@@ -738,15 +803,77 @@ void remmina_pref_init(void)
 	else
 		remmina_pref.vte_shortcutkey_search_text = GDK_KEY_g;
 
-
-	remmina_pref_file_load_colors(gkeyfile, &remmina_pref.color_pref);
-
-	/* If we have a color scheme file, we switch to it, GIO will merge it in the
-	 * remmina.pref file */
-	if (g_file_test(remmina_colors_file, G_FILE_TEST_IS_REGULAR)) {
-		g_key_file_load_from_file(gkeyfile, remmina_colors_file, G_KEY_FILE_NONE, NULL);
-		g_remove(remmina_colors_file);
+	if (g_key_file_has_key(gkeyfile, "remmina_info", "periodic_news_last_checksum", NULL)) {
+		remmina_pref.periodic_news_last_checksum = g_key_file_get_string(gkeyfile, "remmina_info", "periodic_news_last_checksum", NULL);
 	}
+	else {
+		remmina_pref.periodic_news_last_checksum = NULL;
+	}
+
+	if (disablenews) {
+		info_disable_news = 1;
+	}
+	else if (g_key_file_has_key(gkeyfile, "remmina_info", "periodic_news_permitted", NULL)) {
+		remmina_pref.disable_news = !g_key_file_get_boolean(gkeyfile, "remmina_info", "periodic_news_permitted", NULL);
+		info_disable_news = remmina_pref.disable_news;
+	}
+	else {
+		remmina_pref.disable_news = TRUE;
+		info_disable_news = remmina_pref.disable_news;
+	}
+
+	if (disablestats) {
+		info_disable_stats = 1;
+	}
+	else if (g_key_file_has_key(gkeyfile, "remmina_info", "periodic_usage_stats_permitted", NULL)) {
+		remmina_pref.disable_stats = !g_key_file_get_boolean(gkeyfile, "remmina_info", "periodic_usage_stats_permitted", NULL);
+		info_disable_stats = remmina_pref.disable_stats;
+	}
+	else {
+		remmina_pref.disable_stats = TRUE;
+		info_disable_stats = remmina_pref.disable_stats;
+	}
+
+	if (g_key_file_has_key(gkeyfile, "remmina_info", "disable_tip", NULL)) {
+		remmina_pref.disable_tip = g_key_file_get_boolean(gkeyfile, "remmina_info", "disable_tip", NULL);
+		info_disable_tip = remmina_pref.disable_tip;
+	}
+	else {
+		remmina_pref.disable_tip = TRUE;
+		info_disable_tip = remmina_pref.disable_tip;
+	}
+
+	if (g_key_file_has_key(gkeyfile, "remmina_info", "ssh_color_file", NULL)) {
+		remmina_pref.color_file = g_key_file_get_string(gkeyfile, "remmina_info", "ssh_color_file", NULL);
+	}
+	else {
+		remmina_pref.color_file = NULL;
+	}
+
+	#ifdef DISABLE_NEWS
+	info_disable_news = 1;
+	remmina_pref.disable_news = TRUE;
+	#endif
+
+	#ifdef DISABLE_STATS
+	info_disable_stats = 1;
+	remmina_pref.disable_stats = TRUE;
+	#endif
+
+	#ifdef DISABLE_TIP
+	info_disable_tip = 1;
+	remmina_pref.disable_tip = TRUE;
+	#endif
+
+
+	if (g_key_file_has_key(gkeyfile, "remmina_info", "info_uid_prefix", NULL)) {
+		remmina_pref.info_uid_prefix = g_key_file_get_string(gkeyfile, "remmina_info", "info_uid_prefix", NULL);
+	}
+	else {
+		remmina_pref.info_uid_prefix = NULL;
+	}
+
+
 
 	/* Default settings */
 	if (!g_key_file_has_key(gkeyfile, "remmina", "name", NULL)) {
@@ -756,6 +883,15 @@ void remmina_pref_init(void)
 		remmina_pref_save();
 	}
 
+	/* If we have a color scheme file, we switch to it, GIO will merge it in the
+	* remmina.pref file */
+	if (g_file_test(remmina_colors_file, G_FILE_TEST_IS_REGULAR)) {
+		g_key_file_load_from_file(gkeyfile, remmina_colors_file, G_KEY_FILE_NONE, NULL);
+		// g_remove(remmina_colors_file);
+	}
+
+	remmina_pref_file_load_colors(gkeyfile, &remmina_pref.color_pref);
+
 	g_key_file_free(gkeyfile);
 
 	if (remmina_pref.secret == NULL)
@@ -821,12 +957,17 @@ gboolean remmina_pref_save(void)
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "audit", remmina_pref.audit);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "trust_all", remmina_pref.trust_all);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "floating_toolbar_placement", remmina_pref.floating_toolbar_placement);
+	g_key_file_set_integer(gkeyfile, "remmina_pref", "floating_toolbar_monitor", remmina_pref.floating_toolbar_monitor);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "toolbar_placement", remmina_pref.toolbar_placement);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "prevent_snap_welcome_message", remmina_pref.prevent_snap_welcome_message);
 	g_key_file_set_string(gkeyfile, "remmina_pref", "last_quickconnect_protocol", remmina_pref.last_quickconnect_protocol);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "fullscreen_on_auto", remmina_pref.fullscreen_on_auto);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "always_show_tab", remmina_pref.always_show_tab);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "always_show_notes", remmina_pref.always_show_notes);
+	g_key_file_set_boolean(gkeyfile, "remmina_pref", "mp_left", remmina_pref.mp_left);
+	g_key_file_set_boolean(gkeyfile, "remmina_pref", "start_fullscreen", remmina_pref.start_fullscreen);
+	g_key_file_set_boolean(gkeyfile, "remmina_pref", "start_dynres", remmina_pref.start_dynres);
+	g_key_file_set_boolean(gkeyfile, "remmina_pref", "toolbar_fix_position_multimon", remmina_pref.toolbar_fix_position_multimon);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "hide_connection_toolbar", remmina_pref.hide_connection_toolbar);
 	g_key_file_set_boolean(gkeyfile, "remmina_pref", "hide_searchbar", remmina_pref.hide_searchbar);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "default_action", remmina_pref.default_action);
@@ -859,6 +1000,7 @@ gboolean remmina_pref_save(void)
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "default_mode", remmina_pref.default_mode);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "tab_mode", remmina_pref.tab_mode);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "fullscreen_toolbar_visibility", remmina_pref.fullscreen_toolbar_visibility);
+	g_key_file_set_integer(gkeyfile, "remmina_pref", "fullscreen_toolbar_delay", remmina_pref.fullscreen_toolbar_delay);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "auto_scroll_step", remmina_pref.auto_scroll_step);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "hostkey", remmina_pref.hostkey);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_fullscreen", remmina_pref.shortcutkey_fullscreen);
@@ -866,6 +1008,7 @@ gboolean remmina_pref_save(void)
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_nexttab", remmina_pref.shortcutkey_nexttab);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_prevtab", remmina_pref.shortcutkey_prevtab);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_scale", remmina_pref.shortcutkey_scale);
+	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_clipboard", remmina_pref.shortcutkey_clipboard);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_grab", remmina_pref.shortcutkey_grab);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_multimon", remmina_pref.shortcutkey_multimon);
 	g_key_file_set_integer(gkeyfile, "remmina_pref", "shortcutkey_viewonly", remmina_pref.shortcutkey_viewonly);
@@ -907,6 +1050,12 @@ gboolean remmina_pref_save(void)
 	g_key_file_set_string(gkeyfile, "ssh_colors", "color13", remmina_pref.color_pref.color13 ? remmina_pref.color_pref.color13 : "");
 	g_key_file_set_string(gkeyfile, "ssh_colors", "color14", remmina_pref.color_pref.color14 ? remmina_pref.color_pref.color14 : "");
 	g_key_file_set_string(gkeyfile, "ssh_colors", "color15", remmina_pref.color_pref.color15 ? remmina_pref.color_pref.color15 : "");
+	g_key_file_set_string(gkeyfile, "remmina_info", "ssh_color_file", remmina_pref.color_file ? remmina_pref.color_file : "");
+	g_key_file_set_boolean(gkeyfile, "remmina_info", "periodic_news_permitted", !remmina_pref.disable_news);
+	g_key_file_set_string(gkeyfile, "remmina_info", "periodic_news_last_checksum", remmina_pref.periodic_news_last_checksum ? remmina_pref.periodic_news_last_checksum: "");
+	g_key_file_set_boolean(gkeyfile, "remmina_info", "periodic_usage_stats_permitted", !remmina_pref.disable_stats);
+	g_key_file_set_string(gkeyfile, "remmina_info", "info_uid_prefix", remmina_pref.info_uid_prefix ? remmina_pref.info_uid_prefix : "");
+	g_key_file_set_boolean(gkeyfile, "remmina_info", "disable_tip", remmina_pref.disable_tip);
 
 	/* Default settings */
 	g_key_file_set_string(gkeyfile, "remmina", "name", "");
