@@ -47,7 +47,7 @@ static RemminaStringList *string_list;
 #define GET_OBJECT(object_name) gtk_builder_get_object(string_list->builder, object_name)
 
 /* Update the buttons state on the items in the TreeModel */
-void remmina_string_list_update_buttons_state(void)
+static void remmina_string_list_update_buttons_state(void)
 {
 	gint items_count = gtk_tree_model_iter_n_children(
 		GTK_TREE_MODEL(string_list->liststore_items), NULL);
